from dataclasses import field, dataclass
from typing import Union

from django_glue.entities.model_object.response_data import GlueModelObjectJsonData, MethodGlueModelObjectJsonData
from django_glue.response.data import GlueJsonData


@dataclass
class GlueQuerySetJsonData(GlueJsonData):
    model_objects: list[GlueModelObjectJsonData] = field(default_factory=list)

    def to_dict(self):
        return [model_object.to_dict() for model_object in self.model_objects]


@dataclass
class MethodGlueQuerySetJsonData(GlueJsonData):
    method_returns: list[MethodGlueModelObjectJsonData] = field(default_factory=list)

    def to_dict(self):
        return [method_return.to_dict() for method_return in self.method_returns]


@dataclass
class ToChoicesGlueQuerySetJsonData(GlueJsonData):
    choices: list[Union[tuple, list]]

    def to_dict(self):
        return self.choices



import json

from django.core.serializers.json import DjangoJSONEncoder

from django_glue.conf import settings
from django_glue import __version__
from django_glue.session import GlueSession


def glue(request):
    return {
        'DJANGO_GLUE_VERSION': __version__,
        'DJANGO_GLUE_KEEP_LIVE_INTERVAL_TIME_MILLISECONDS': (settings.DJANGO_GLUE_KEEP_LIVE_EXPIRE_TIME_SECONDS / 2.2) * 1000,
        'GLUE_SESSION_DATA': json.dumps(GlueSession(request).session, cls=DjangoJSONEncoder),
    }



# Generated by Django 5.0.6 on 2024-05-25 14:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0003_testmodel_email_testmodel_personality_type_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='testmodel',
            name='likes_to_party',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='testmodel',
            name='personality_type',
            field=models.CharField(choices=[('int', 'Introvert'), ('ext', 'Extrovert')], default='int', max_length=3),
        ),
    ]



from django_glue.entities.template.actions import GlueTemplateAction
from django_glue.entities.template import handlers


GLUE_TEMPLATE_HANDLER_MAP = {
    GlueTemplateAction.GET: handlers.GetGlueTemplateHandler,
}



# Generated by Django 4.0.5 on 2023-08-03 17:38

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='bigtestmodel',
            name='foreign_key',
            field=models.ForeignKey(default=271, on_delete=django.db.models.deletion.CASCADE, to='tests.testmodel'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='testmodel',
            name='birth_date',
            field=models.DateField(default=django.utils.timezone.localdate),
        ),
    ]



from django.test import TestCase
from django.urls import reverse, get_resolver


class TestViews(TestCase):
    def test_model_object_view(self):
        print(get_resolver().reverse_dict.keys())
        response = self.client.get(reverse('model_object'))
        self.assertTrue(response.status_code == 200)

    def test_function_view(self):
        print(get_resolver().reverse_dict.keys())
        response = self.client.get(reverse('function'))
        self.assertTrue(response.status_code == 200)

    def test_template_view(self):
        print(get_resolver().reverse_dict.keys())
        response = self.client.get(reverse('template'))
        self.assertTrue(response.status_code == 200)

    def test_view_view(self):
        print(get_resolver().reverse_dict.keys())
        response = self.client.get(reverse('view'))
        self.assertTrue(response.status_code == 200)


from dataclasses import dataclass, field
from typing import Optional

from django_glue.form.field.attrs.entities import GlueFieldAttrs


@dataclass
class GlueFormField:
    name: str
    type: str
    attrs: GlueFieldAttrs
    label: Optional[str] = None
    id: Optional[str] = None
    help_text: str = ''
    choices: Optional[list] = field(default_factory=list)

    def __post_init__(self):
        if self.id is None:
            self.id = f'id_{self.name}'

        if self.label is None:
            self.label = ' '.join(self.name.split('_')).title()

    def to_dict(self) -> dict:
        return {
            'name': self.name,
            'type': self.type,
            'attrs': self.attrs.to_dict(),
            'label': self.label,
            'help_text': self.help_text,
            'choices': self.choices
        }



from __future__ import annotations

from enum import Enum

from django_glue.access.access import GlueAccess


class GlueAction(str, Enum):

    def __str__(self):
        return self.value

    def required_access(self) -> GlueAccess:
        raise NotImplemented('You must override required access on Glue Actions')


import json

from django.core.serializers.json import DjangoJSONEncoder

from django_glue.conf import settings

from django_glue.session.data import GlueSessionData
from django_glue.session.session import Session


class GlueSession(Session):
    """
        Used to add models, query sets, and other objects to the session.
    """
    def __init__(self, request):
        super().__init__(request)
        self.request.session.setdefault(settings.DJANGO_GLUE_SESSION_NAME, dict())
        self.session = self.request.session[settings.DJANGO_GLUE_SESSION_NAME]

    def __getitem__(self, key):
        return self.session[key]

    def __setitem__(self, key, value):
        self.session[key] = value

    def add_glue_entity(self, glue_entity: 'GlueEntity'):
        if glue_entity.unique_name in self.session:
            self.purge_unique_name(glue_entity.unique_name)

        self.add_session_data(glue_entity.unique_name, glue_entity.to_session_data())
        self.set_modified()

    def add_session_data(self, unique_name, session_data: GlueSessionData) -> None:
        self.session[unique_name] = session_data.to_dict()

    def clean(self, removable_unique_names):
        for unique_name in removable_unique_names:
            self.purge_unique_name(unique_name)

        self.set_modified()

    def purge_unique_name(self, unique_name):
        self.session.pop(unique_name)

    def set_modified(self):
        self.request.session.modified = True

    def to_json(self):
        return json.dumps(self.session, cls=DjangoJSONEncoder)



from enum import Enum


class GlueJsonResponseType(str, Enum):
    SUCCESS = 'success'
    INFO = 'info'
    WARNING = 'warning'
    ERROR = 'error'
    DEBUG = 'debug'

    def __str__(self):
        return self.value


class GlueJsonResponseStatus(str, Enum):
    SUCCESS = '200'
    SILENT_SUCCESS = '204'
    ERROR = '404'

    def __str__(self):
        return self.value




from dataclasses import dataclass
from typing import Any, Optional, Union


# Model objects can be used from a queryset. Queryset and model object post data must be the same.
@dataclass
class BasePostData:
    id: int

    # def __post_init__(self):
    #     if isinstance(self.id, int):
    #         self.id = [self.id]
    #     elif isinstance(self.id, str):
    #         self.id = [int(self.id)]


@dataclass
class DeletePostData(BasePostData):
    pass


@dataclass
class GetPostData(BasePostData):
    pass


@dataclass
class MethodPostData(BasePostData):
    method: str
    kwargs: dict[str, Any]


@dataclass
class UpdatePostData(BasePostData):
    fields: dict[str, Any]



from django.core import serializers


def camel_to_snake(string):
    return ''.join(['_' + c.lower() if c.isupper() else c for c in string]).lstrip('_')


def serialize_object_to_json(model_object):
    return serializers.serialize('json', [model_object, ])



from __future__ import annotations

import json
from abc import ABC, abstractmethod
from typing import Any, Optional
from dataclasses import dataclass, asdict

from django.core.serializers.json import DjangoJSONEncoder
from django.http import JsonResponse

from django_glue.response.enums import GlueJsonResponseType, GlueJsonResponseStatus


@dataclass
class GlueJsonData(ABC):

    @abstractmethod
    def to_dict(self):
        return asdict(self)

    def to_json(self):
        return json.dumps(self.to_dict(), cls=DjangoJSONEncoder)


@dataclass
class GlueJsonResponseData:
    """
        Consistent structure for our json responses.
    """
    message_title: Optional[str] = None
    message_body: Optional[str] = None
    data: Optional[GlueJsonData] = None
    optional_message_data: Optional[dict] = None
    response_type: GlueJsonResponseType = GlueJsonResponseType.SUCCESS
    response_status: GlueJsonResponseStatus = GlueJsonResponseStatus.SUCCESS

    def to_dict(self) -> dict:

        if isinstance(self.data, GlueJsonData):
            self.data = self.data.to_json()

        json_response_dict = asdict(self)

        return json_response_dict

    def to_django_json_response(self) -> JsonResponse:
        return JsonResponse(self.to_dict(), status=self.response_status.value)



from django_glue.entities.function.actions import GlueFunctionAction
from django_glue.entities.function import handlers


GLUE_FUNCTION_HANDLER_MAP = {
    GlueFunctionAction.CALL: handlers.CallGlueFunctionHandler,
}



# Generated by Django 5.0.6 on 2024-05-25 16:07

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0004_testmodel_likes_to_party_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='testmodel',
            name='bed_time',
            field=models.TimeField(default=django.utils.timezone.now),
        ),
    ]



from django_glue.handler.body_data import GlueBodyData
from django_glue.handler.enums import GlueConnection
from django_glue.response.data import GlueJsonResponseData

from django_glue.handler.maps import CONNECTION_TO_HANDLER_MAP
from django_glue.session import GlueSession


def process_glue_request(glue_session: GlueSession, glue_body_data: GlueBodyData) -> GlueJsonResponseData:
    # Todo: Validation errors.
    connection = GlueConnection(glue_session[glue_body_data.unique_name]['connection'])

    handler_class = CONNECTION_TO_HANDLER_MAP[GlueConnection(connection)][glue_body_data.action]
    return handler_class(
        glue_session=glue_session,
        glue_body_data=glue_body_data
    ).process_response_data()



from dataclasses import dataclass
from typing import Any

from django_glue.response.data import GlueJsonData


@dataclass
class GlueFunctionJsonData(GlueJsonData):
    function_return: Any

    def to_dict(self):
        return {'function_return': self.function_return}


from django.urls import resolve

from django_glue.session import GlueSession, GlueKeepLiveSession


class GlueMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        current_url = resolve(request.path_info).url_name

        if current_url not in ['django_glue_data_handler', 'django_glue_keep_live_handler']:
            glue_session = GlueSession(request)
            glue_keep_live_session = GlueKeepLiveSession(request)
            glue_session.clean(glue_keep_live_session.clean_and_get_expired_unique_names())

        response = self.get_response(request)

        return response



from django_glue.access.decorators import check_access
from django_glue.entities.template.actions import GlueTemplateAction
from django_glue.entities.template.post_data import GetGlueTemplatePostData

from django_glue.entities.template.session_data import TemplateSessionData
from django_glue.entities.template.entities import GlueTemplate
from django_glue.handler.handlers import GlueRequestHandler
from django_glue.response.data import GlueJsonResponseData
from django_glue.response.responses import generate_json_200_response_data


class GetGlueTemplateHandler(GlueRequestHandler):
    action = GlueTemplateAction.GET
    _session_data_class = TemplateSessionData
    _post_data_class = GetGlueTemplatePostData

    @check_access
    def process_response_data(self) -> GlueJsonResponseData:
        glue_template = GlueTemplate(
            unique_name=self.session_data.unique_name,
            template_name=self.session_data.template_name
        )
        rendered_template = glue_template.render_to_string(self.post_data.context_data)
        return generate_json_200_response_data(
            'THE METHOD ACTION',
            'this is a response from an model object method action!',
            data=glue_template.to_response_data(rendered_template).to_dict()
        )



from django_glue.entities.model_object.actions import GlueModelObjectAction
from django_glue.entities.model_object import handlers


GLUE_MODEL_OBJECT_HANDLER_MAP = {
    GlueModelObjectAction.GET: handlers.GetGlueModelObjectHandler,
    GlueModelObjectAction.UPDATE: handlers.UpdateGlueModelObjectHandler,
    GlueModelObjectAction.METHOD: handlers.MethodGlueModelObjectHandler,
    GlueModelObjectAction.DELETE: handlers.DeleteGlueModelObjectHandler
}



import unittest


class TestCore(unittest.TestCase):
    def test_everything(self):
        try:
            self.assertTrue(True)
        except:
            self.assertTrue(False)


if __name__ == '__main__':
    unittest.main()



from django_glue.access.decorators import check_access
from django_glue.entities.model_object.factories import glue_model_object_from_glue_query_set_session, \
    glue_model_objects_from_query_set
from django_glue.entities.model_object.response_data import GlueModelObjectJsonData, MethodGlueModelObjectJsonData
from django_glue.entities.query_set.actions import GlueQuerySetAction
from django_glue.entities.query_set.factories import glue_query_set_from_session_data
from django_glue.entities.post_data import GetPostData, DeletePostData, UpdatePostData, MethodPostData
from django_glue.entities.query_set.post_data import FilterGlueQuerySetPostData
from django_glue.entities.query_set.response_data import GlueQuerySetJsonData, MethodGlueQuerySetJsonData, \
    ToChoicesGlueQuerySetJsonData

from django_glue.entities.query_set.session_data import GlueQuerySetSessionData
from django_glue.handler.handlers import GlueRequestHandler
from django_glue.response.data import GlueJsonResponseData
from django_glue.response.responses import generate_json_200_response_data


class AllGlueQuerySetHandler(GlueRequestHandler):
    action = GlueQuerySetAction.ALL
    _session_data_class = GlueQuerySetSessionData

    @check_access
    def process_response_data(self) -> GlueJsonResponseData:

        glue_query_set = glue_query_set_from_session_data(self.session_data)
        glue_model_objects = glue_model_objects_from_query_set(glue_query_set.query_set.all(), self.session_data)

        return generate_json_200_response_data(
            message_title='Success',
            message_body='Successfully retrieved model object!',
            data=GlueQuerySetJsonData([GlueModelObjectJsonData(glue_model_object.fields) for glue_model_object in glue_model_objects])
        )


class DeleteGlueQuerySetHandler(GlueRequestHandler):
    action = GlueQuerySetAction.DELETE
    _session_data_class = GlueQuerySetSessionData
    _post_data_class = DeletePostData

    @check_access
    def process_response_data(self) -> GlueJsonResponseData:
        glue_query_set = glue_query_set_from_session_data(self.session_data)

        filtered_query_set = glue_query_set.query_set.filter(id__in=self.post_data.id)
        filtered_query_set.delete()

        return generate_json_200_response_data(
            message_title='Success',
            message_body='Successfully deleted queryset!',
        )


class FilterGlueQuerySetHandler(GlueRequestHandler):
    action = GlueQuerySetAction.FILTER
    _session_data_class = GlueQuerySetSessionData
    _post_data_class = FilterGlueQuerySetPostData

    @check_access
    def process_response_data(self) -> GlueJsonResponseData:

        glue_query_set = glue_query_set_from_session_data(self.session_data)
        filtered_query_set = glue_query_set.query_set.filter(**self.post_data.filter_params)
        glue_model_objects = glue_model_objects_from_query_set(filtered_query_set, self.session_data)

        return generate_json_200_response_data(
            message_title='Success',
            message_body='Successfully retrieved model object!',
            data=GlueQuerySetJsonData([GlueModelObjectJsonData(glue_model_object.fields) for glue_model_object in glue_model_objects])
        )


class GetGlueQuerySetHandler(GlueRequestHandler):
    action = GlueQuerySetAction.GET
    _session_data_class = GlueQuerySetSessionData
    _post_data_class = GetPostData

    @check_access
    def process_response_data(self) -> GlueJsonResponseData:
        glue_query_set = glue_query_set_from_session_data(self.session_data)

        model_object = glue_query_set.query_set.get(id=self.post_data.id)
        glue_model_object = glue_model_object_from_glue_query_set_session(model_object, self.session_data)

        return generate_json_200_response_data(
            message_title='Success',
            message_body='Successfully retrieved model object!',
            data=GlueQuerySetJsonData([GlueModelObjectJsonData(glue_model_object.fields)])
        )


class NullObjectGlueQuerySetHandler(GlueRequestHandler):
    action = GlueQuerySetAction.NULL_OBJECT
    _session_data_class = GlueQuerySetSessionData

    @check_access
    def process_response_data(self) -> GlueJsonResponseData:
        glue_query_set = glue_query_set_from_session_data(self.session_data)
        glue_model_object = glue_model_object_from_glue_query_set_session(glue_query_set.model(), self.session_data)

        return generate_json_200_response_data(
            message_title='Success',
            message_body='Successfully retrieved model object!',
            data=GlueQuerySetJsonData([GlueModelObjectJsonData(glue_model_object.fields)])
        )


class MethodGlueQuerySetHandler(GlueRequestHandler):
    action = GlueQuerySetAction.METHOD
    _session_data_class = GlueQuerySetSessionData
    _post_data_class = MethodPostData

    def process_response_data(self) -> GlueJsonResponseData:
        glue_query_set = glue_query_set_from_session_data(self.session_data)

        if isinstance(self.post_data.id, int):
            # Called from a glue queryset on a model object
            model_object = glue_query_set.query_set.get(id=self.post_data.id)
            glue_model_object = glue_model_object_from_glue_query_set_session(model_object, self.session_data)
            method_return = glue_model_object.call_method(self.post_data.method, self.post_data.kwargs)

            return generate_json_200_response_data(
                message_title='Success',
                message_body='Successfully updated model object!',
                data=MethodGlueModelObjectJsonData(method_return)
            )
        else:
            # Called from a glue queryset
            filtered_query_set = glue_query_set.query_set.filter(id__in=self.post_data.id)

            method_return_data = []

            for model_object in filtered_query_set:
                glue_model_object = glue_model_object_from_glue_query_set_session(model_object, self.session_data)
                method_return = glue_model_object.call_method(self.post_data.method, self.post_data.kwargs)
                method_return_data.append(MethodGlueModelObjectJsonData(method_return))

            return generate_json_200_response_data(
                message_title='Success',
                message_body='Successfully updated model object!',
                data=MethodGlueQuerySetJsonData(method_return_data)
            )


class UpdateGlueQuerySetHandler(GlueRequestHandler):
    action = GlueQuerySetAction.UPDATE
    _session_data_class = GlueQuerySetSessionData
    _post_data_class = UpdatePostData

    @check_access
    def process_response_data(self) -> GlueJsonResponseData:
        glue_query_set = glue_query_set_from_session_data(self.session_data)

        model_object = glue_query_set.query_set.get(id=self.post_data.id)

        glue_model_object = glue_model_object_from_glue_query_set_session(model_object, self.session_data)
        glue_model_object.update(self.post_data.fields)

        return generate_json_200_response_data(
            message_title='Success',
            message_body='Successfully updated model object!',
            data=GlueQuerySetJsonData([GlueModelObjectJsonData(glue_model_object.fields)])
        )


class ToChoicesGlueQuerySetHandler(GlueRequestHandler):
    action = GlueQuerySetAction.TO_CHOICES
    _session_data_class = GlueQuerySetSessionData
    _post_data_class = FilterGlueQuerySetPostData

    @check_access
    def process_response_data(self) -> GlueJsonResponseData:
        glue_query_set = glue_query_set_from_session_data(self.session_data)
        filtered_query_set = glue_query_set.query_set.filter(**self.post_data.filter_params)
        glue_model_objects = glue_model_objects_from_query_set(filtered_query_set, self.session_data)
        choices = [(glue_model_obj.model_object.pk, str(glue_model_obj.model_object)) for glue_model_obj in glue_model_objects]

        return generate_json_200_response_data(
            message_title='Success',
            message_body='Successfully retrieved model object!',
            data=ToChoicesGlueQuerySetJsonData(choices)
        )



# Generated by Django 4.0.4 on 2022-08-31 03:50

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BigTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('big_integer_field', models.BigIntegerField()),
                ('binary_field', models.BinaryField()),
                ('boolean_field', models.BooleanField()),
                ('char_field', models.CharField(max_length=8)),
                ('date_field', models.DateField()),
                ('date_time_field', models.DateTimeField()),
                ('decimal_field', models.DecimalField(decimal_places=2, max_digits=6)),
                ('duration_field', models.DurationField()),
                ('email_field', models.EmailField(max_length=254)),
                ('file_path_field', models.FilePathField(path='/static')),
                ('float_field', models.FloatField()),
                ('generic_ip_address_field', models.GenericIPAddressField()),
                ('ip_address_field', models.GenericIPAddressField()),
                ('integer_field', models.IntegerField()),
                ('positive_big_integer_field', models.PositiveBigIntegerField()),
                ('positive_integer_field', models.PositiveIntegerField()),
                ('positive_small_integer_field', models.PositiveSmallIntegerField()),
                ('slug_field', models.SlugField()),
                ('small_integer_field', models.SmallIntegerField()),
                ('text_field', models.TextField()),
                ('time_field', models.TimeField()),
                ('url_field', models.URLField()),
                ('uuid_field', models.UUIDField()),
            ],
        ),
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=32)),
                ('last_name', models.CharField(max_length=32)),
                ('description', models.TextField()),
                ('favorite_number', models.IntegerField()),
                ('anniversary_datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('birth_date', models.DateField(default=django.utils.timezone.now)),
                ('weight_lbs', models.DecimalField(decimal_places=3, max_digits=7)),
            ],
        ),
    ]



import json


class GlueBodyData:
    def __init__(self, request_body):
        self.data = json.loads(request_body.decode('utf-8'))
        self.unique_name = self.data['unique_name']
        self.action = self.data['action']

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        self.data[key] = value



from time import time

from django_glue.conf import settings
from django_glue.session.session import Session


class GlueKeepLiveSession(Session):
    """
        Used to keep glue session data live for a set amount of time.
        Functionality to handle multiple windows/tabs.
    """
    def __init__(self, request):
        super().__init__(request)
        self.session = request.session.setdefault(settings.DJANGO_GLUE_KEEP_LIVE_SESSION_NAME, dict())

    def __getitem__(self, key):
        return self.session[key]

    def __setitem__(self, key, value):
        self.session[key] = value

    def clean_and_get_expired_unique_names(self) -> list:
        expired_unique_names = []

        for key, val in self.session.items():
            if time() > val:
                expired_unique_names.append(key)

        for expired_unique_name in expired_unique_names:
            self.session.pop(expired_unique_name)

        self.set_modified()

        return expired_unique_names

    @staticmethod
    def get_next_expire_time():
        return time() + settings.DJANGO_GLUE_KEEP_LIVE_EXPIRE_TIME_SECONDS

    def set_unique_name(self, unique_name):
        self.session.setdefault(unique_name, self.get_next_expire_time())

        self.set_modified()

    def set_modified(self):
        self.request.session.modified = True

    def update_unique_names(self, unique_names: list):
        for unique_name in unique_names:
            if unique_name in self.session:
                self.session[unique_name] = self.get_next_expire_time()

        self.set_modified()



from typing import Type

from django.db.models import Model

from django_glue.entities.model_object.fields.entities import GlueModelFields, GlueModelField, GlueModelFieldMeta
from django_glue.entities.model_object.fields.utils import field_name_included
from django_glue.form.field.factories import GlueFormFieldFactory


def model_object_fields_from_model(
        model: Type[Model],
        included_fields: tuple,
        excluded_fields: tuple
) -> GlueModelFields:
    fields = []

    for model_field in model._meta.fields:
        if field_name_included(model_field.name, included_fields, excluded_fields):

            _meta = GlueModelFieldMeta(
                type=model_field.get_internal_type(),
                name=model_field.name,
                glue_field=GlueFormFieldFactory(model_field).factory_method()
            )

            glue_model_field = GlueModelField(
                name=model_field.name,
                value=None,
                _meta=_meta
            )

            fields.append(glue_model_field)

    return GlueModelFields(fields=fields)



from dataclasses import dataclass

from django_glue.response.data import GlueJsonData


@dataclass
class GlueTemplateJsonData(GlueJsonData):
    rendered_template: str

    def to_dict(self):
        return {'rendered_template': self.rendered_template}





# Generated by Django 5.0.6 on 2024-08-09 21:38

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0006_uuidtestmodel_alter_testmodel_bed_time'),
    ]

    operations = [
        migrations.AddField(
            model_name='testmodel',
            name='best_friend',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='tests.testmodel'),
        ),
    ]



import logging
from dataclasses import field

from django.db.models import Model

from django_glue.entities.model_object.fields.entities import GlueModelField


def field_name_included(name, fields, exclude):
    included = False
    if name not in exclude or exclude[0] == '__none__':
        if name in fields or fields[0] == '__all__':
            included = True

    return included


def get_field_value_from_model_object(model_object: Model, field: GlueModelField):
    relational = ['ForeignKey', 'BinaryField', 'OnetoOneField']
    if field._meta.type in relational:
        return getattr(model_object, f'{field.name}_id')
    else:
        return getattr(model_object, field.name)



import json
from abc import ABC
from dataclasses import dataclass, asdict

from django.core.serializers.json import DjangoJSONEncoder

from django_glue.access.access import GlueAccess
from django_glue.handler.enums import GlueConnection


@dataclass
class GlueSessionData(ABC):
    unique_name: str
    connection: GlueConnection
    access: GlueAccess

    def to_dict(self) -> dict:
        return asdict(self)

    def to_json(self) -> str:
        return json.dumps(self.to_dict(), cls=DjangoJSONEncoder)


from dataclasses import dataclass, field
from typing import Union

from django_glue.session.data import GlueSessionData


@dataclass
class GlueQuerySetSessionData(GlueSessionData):
    query_set_str: str
    fields: dict
    included_fields: Union[list, tuple]
    excluded_fields: Union[list, tuple]
    included_methods: Union[list, tuple]



from django_glue.access.access import GlueAccess
from django_glue.access.actions import GlueAction


class GlueTemplateAction(GlueAction):
    GET = 'get'

    def required_access(self) -> GlueAccess:
        if self.value == 'get':
            return GlueAccess.VIEW
        else:
            raise ValueError('That is not a valid action on a glue query set.')



from django.contrib import admin
from django.urls import path, include
from django.views.generic import TemplateView

from tests import views

urlpatterns = [
    path("", views.ModelObjectView.as_view(), name="model_object"),
    path("big_model", views.big_model_object_view, name="big_model"),
    path("query_set/", views.QuerySetView.as_view(), name="query_set"),
    path("query_set/list/", views.query_set_list_view, name="query_set_list"),
    path("other/", views.OtherView.as_view(), name="other_glue"),
    path("no_glue/", TemplateView.as_view(template_name='page/no_glue_page.html'), name="no_glue"),
    path("benchmark/", views.benchmark_view, name="benchmark"),
    path("django_glue/", include('django_glue.urls', namespace='django_glue')),
    path("template/", views.template_view, name="template"),
    path("view/", views.view_view, name="view"),
    path("view/card/", views.view_card_view, name="view_card"),
    path("function/", views.function_view, name="function"),
    path("form/fields/", views.form_field_view, name="form_fields"),
    path("complex_form/", views.complex_form_view, name="complex_form"),
    path("complex_model_form/", views.complex_model_form_view, name="complex_model_form"),
    path("uuid_model/", views.uuid_model_view, name="uuid_model"),
]

urlpatterns += [
    path('admin/', admin.site.urls),
]


import uuid
from dataclasses import dataclass
from typing import Union

from django_glue.entities.model_object.fields.entities import GlueModelField
from django_glue.session.data import GlueSessionData


@dataclass
class GlueModelObjectSessionData(GlueSessionData):
    app_label: str
    model_name: str
    object_pk: Union[int, str, uuid.uuid4]
    fields: [GlueModelField]
    included_fields: Union[list, tuple]
    exclude_fields: Union[list, tuple]
    methods: Union[list, tuple]

    def __post_init__(self):
        if isinstance(self.object_pk, uuid.UUID):
            self.object_pk = str(self.object_pk)




from django_glue.access.decorators import check_access
from django_glue.entities.function.actions import GlueFunctionAction
from django_glue.entities.function.entities import GlueFunction
from django_glue.entities.function.post_data import CallGlueFunctionPostData

from django_glue.entities.function.session_data import FunctionSessionData
from django_glue.handler.handlers import GlueRequestHandler
from django_glue.response.data import GlueJsonResponseData
from django_glue.response.responses import generate_json_200_response_data


class CallGlueFunctionHandler(GlueRequestHandler):
    action = GlueFunctionAction.CALL
    _session_data_class = FunctionSessionData
    _post_data_class = CallGlueFunctionPostData

    @check_access
    def process_response_data(self) -> GlueJsonResponseData:
        glue_function = GlueFunction(
            unique_name=self.session_data.unique_name,
            function_path=self.session_data.function_path
        )
        function_return = glue_function.call(self.post_data.kwargs)
        return generate_json_200_response_data(
            message_title='Success',
            message_body='Successfully retrieved model object!',
            data=glue_function.to_response_data(function_return)
        )




from dataclasses import dataclass, field
from typing import Any, Union

from django_glue.entities.model_object.fields.seralizers import serialize_field_value
from django_glue.form.field.entities import GlueFormField


@dataclass
class GlueModelFieldMeta:
    type: str
    name: str
    glue_field: GlueFormField

    def to_dict(self) -> dict:
        return {
            'type': self.type,
            'name': self.name,
            'glue_field': self.glue_field.to_dict(),
        }


@dataclass
class GlueModelField:
    name: str
    value: Any
    _meta: Union[GlueModelFieldMeta, dict] = field(default_factory=dict)

    def to_dict(self) -> dict:
        return {
            'name': self.name,
            'value': serialize_field_value(self),
            '_meta': self._meta.to_dict()
        }


@dataclass
class GlueModelFields:
    fields: list[GlueModelField] = field(default_factory=list)

    def __iter__(self):
        return self.fields.__iter__()

    def to_dict(self):
        # Todo: Should this just be a list of fields?
        return {field.name: field.to_dict() for field in self.fields}
